@@ -22,12 +22,17 @@ Student Intervention widgets.
 from zope.browser.interfaces import IAdding 
 from zope.formlib.interfaces import MissingInputError
 from zope.formlib.interfaces import IBrowserWidget, IInputWidget
+from zope.formlib.interfaces import InputErrors
 from zope.app.form.browser.submit import Update
 from zope.browserpage import ViewPageTemplateFile
-from zope.component import getUtility, queryUtility
-from zope.interface import implements
+from zope.component import adapter, adapts, getUtility, queryUtility
+from zope.interface import implementer, implements
 from zope.security.proxy import removeSecurityProxy
 
+from z3c.form.converter import BaseDataConverter
+from z3c.form.interfaces import IWidget, IFieldWidget, IDataConverter
+from z3c.form.widget import Widget, FieldWidget
+
 from schooltool.contact.interfaces import IContactable, IContact, IUniqueFormKey
 from schooltool.contact.browser.relationship import get_relationship_title
 from schooltool.course.interfaces import ISectionContainer
@@ -38,7 +43,7 @@ from schooltool.schoolyear.interfaces import ISchoolYear
 from schooltool.term.interfaces import IDateManager
 
 from schooltool.intervention import InterventionGettext as _
-from schooltool.intervention import intervention
+from schooltool.intervention import interfaces, intervention
 
 
 class WidgetBase(object):
@@ -217,12 +222,139 @@ class PersonListWidget(WidgetBase):
         contacts = self.getRequestValue()
         if not contacts and self.enabled:
             self._error = MissingInputError.__doc__
-            raise MissingInputError('', '')
+            raise MissingInputError(self.name, self.label, None)
         return contacts
 
     def hasValidInput(self):
         """See zope.formlib.interfaces.IInputWidget"""
-        return bool(self.getInputValue())
+        try:
+            self.getInputValue()
+            return True
+        except InputErrors:
+            return False
+
+
+class PersonListZ3CFormWidget(Widget, PersonListWidget):
+    """z3c form widget for PersonList field."""
+
+    template = ViewPageTemplateFile('templates/f_person_list.pt')
+
+    def __getitem__(self, attr):
+        return getattr(self, attr)
+
+    def addClass(self, klass):
+        pass
+
+    def update(self):
+        """See z3c.form.interfaces.IWidget."""
+        self.student = IPerson(self.context)
+
+        # establish email capability and error message
+        utility = getUtility(IEmailUtility)
+        self.enabled = utility.enabled()
+        self.message = ""
+        if not self.enabled:
+            self.message = _('Email server is not enabled')
+        else:
+            sender = IContact(self.request.principal._person)
+            if not sender.email:
+                self.message = _('User has no email address')
+                self.enabled = False
+
+        # build responsible, notified and changed lists of contacts
+        term = queryUtility(IDateManager).current_term
+        advisors = [IContact(removeSecurityProxy(advisor))
+                    for advisor in self.student.advisors]
+        groups = IGroupContainer(ISchoolYear(term))
+        administrators = [IContact(member)
+                          for member in groups['administrators'].members]
+        responsible = advisors + [contact for contact in administrators
+                                  if contact not in advisors]
+        for section in ISectionContainer(term).values():
+            if self.student in section.members:
+                for teacher in section.instructors:
+                    if IContact(teacher) not in responsible:
+                        responsible.append(IContact(teacher))
+        notified = [IContact(self.student)]
+        notified += [contact for contact in IContactable(self.student).contacts]
+        if self.isAddView():
+            changed = []
+        else:
+            changed = [contact
+                       for contact in self.context.at_one_time_responsible
+                       if contact not in responsible + notified]
+
+        # set choices attribute with complete list of contact choices
+        self.choices = responsible + notified + changed
+
+        # build value list according to view type, request
+        valueList = self.getRequestValue()
+        if not self.isAddView() and not 'SUBMIT' in self.request:
+            valueList = getattr(self.context, self.field.getName())
+
+        # build final view attributes: responsible, notified and changed
+        self.responsible = []
+        self.notified = []
+        self.changed = []
+        for choice in self.choices:
+            checked = ''
+            if choice in valueList:
+                checked = 'checked'
+
+            value = intervention.contactName(choice)
+            if choice in advisors:
+                value = '%s (%s)' % (value, _('advisor'))
+                if self.isAddView() and not 'SUBMIT' in self.request:
+                    checked = 'checked'
+
+            if choice in IContactable(self.student).contacts:
+                title = get_relationship_title(self.student, choice)
+                value = '%s (%s)' % (value, title.lower())
+
+            person = {
+                'name': IUniqueFormKey(choice),
+                'checked': checked,
+                'value': value,
+                'hasEmail': bool(removeSecurityProxy(choice).email),
+                }
+
+            if choice in notified:
+                self.notified.append(person)
+            elif choice in changed:
+                self.changed.append(person)
+            else:
+                self.responsible.append(person)
+
+        # finally call the z3c base class
+        super(PersonListZ3CFormWidget, self).update()
+
+    def extract(self):
+        return self.getRequestValue()
+
+    def isAddView(self):
+        return not interfaces.IInterventionMarker.providedBy(self.context)
+
+    def render(self):
+        self.update()
+        return self.template()
+
+
+def PersonListFieldWidget(field, request):
+    """IFieldWidget factory for MyWidget."""
+    widget = PersonListZ3CFormWidget(request)
+    return FieldWidget(field, removeSecurityProxy(widget))
+
+
+class PersonListFieldDataConverter(BaseDataConverter):
+    """A PersonListField data converter."""
+    adapts(interfaces.IPersonListField, IFieldWidget)
+    implements(IDataConverter)
+
+    def toFieldValue(self, value):
+        """See interfaces.IDataConverter"""
+        if value == []:
+            return self.field.missing_value
+        return value
 
 
 class GoalMetWidget(WidgetBase):
